#!/dcs/packages/python-2.1/bin/python

#dialog version 0.3, by Savio Lam (lam836@cs.cuhk.hk).
#  patched to version 0.6gnome1 by Stuart Herbert (S.Herbert@shef.ac.uk)
#
#* Display dialog boxes from shell scripts *
#
#Usage: gdialog --clear
#       gdialog --create-rc <file>
#       gdialog [--title <title>] [--separate-output] [--backtitle <backtitle>] [--clear] <Box options>
#
#Box options:
#
#  [--defaultno] --yesno     <text> <height> <width> [--defaultno]
#  --msgbox    <text> <height> <width>
#  --infobox   <text> <height> <width>
#  --inputbox  <text> <height> <width> [<init>]
#  --textbox   <file> <height> <width>
#  --menu      <text> <height> <width> <menu height> <tag1> <item1>...
#  --checklist <text> <height> <width> <list height> <tag1> <item1> <status1>...
#  --radiolist <text> <height> <width> <list height> <tag1> <item1> <status1>...
#  --gauge     <text> <height> <width> <percent>
#  --guage     <text> <height> <width> <percent>

import os
import string
import sys
import time

class pdialog:

	def initfailed(self):
		sys.stderr.write('pdialog: could not initialize.  Reason: \n')
		sys.stderr.write('(X)dialog not found on path.\n')
		sys.exit(1)

	def __init__(self):
		self.x = 1
		disp=os.getenv('DISPLAY')	
		if disp != '':
			self.x = 1
			if os.system('xdpyinfo > /dev/null 2>&1') != 0:
				# Sometimes we'll have a $DISPLAY, but we still cannot run X
				# commands.
				# Note we're using shell true in the if above, not python true.
				# Also note that the return value is the shell return value
				# times 256
				self.x = 0
				sys.stderr.write('Warning: no X11 available!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\007\n')
			else:
				# we know we have X, but do we have Xdialog?  If yes, then
				# use it, if no, then run dialog in a terminal emulator,
				# like xterm
				self.prog = self.__findexe('Xdialog')
				if self.prog == '':
					# nope, no Xdialog.  Try to fake it
					dialog_path = self.__findexe('dialog')
					if dialog_path != '':
						term_path = self.__findexe('mrxvt')
						if term_path == '':
							term_path = self.__findexe('rxvt')
						if term_path == '':
							term_path = self.__findexe('konsole')
						if term_path == '':
							term_path = self.__findexe('xterm')
						if term_path == '':
							term_path = self.__findexe('gnome-terminal')
						if term_path == '':
							self.initfailed()
						self.prog="%s -e %s" % (term_path, dialog_path)
						sys.stderr.write('pdialog using program %s\n' % self.prog)
		else:
			self.x = 0
		if self.x == 0:
			self.prog=self.__findexe('dialog')
		if self.prog == '':
			self.initfailed()

	def __findexe(self,exe):
		p=os.getenv('PATH')
		for dir in string.splitfields(p,':'):
			full = dir+'/'+exe
			if os.access(full,1):
				return full
		return ''

	def __scale(self,i):
		return str(i)
		# this is important with gdialog, but not with Xdialog
		#if self.x:
		#	return str(i)
		#else:
		#	return str(i/8)

	def __nonl(self,str):
		if str[-1:] == '\n':
			return str[:-1]
		else:
			return str

#	def accident_proof_acknowledge(self,text,width,height):
#		#dialog  --defaultno --yesno 'I know the process has completed' 30 80 ; echo $?
#		s=self.prog+' --defaultno --yesno \''+text+'\' '+\
#			self.__scale(height)+' '+self.__scale(width)
#		# keep looping until the user selects "yes".  Default is no.  This
#		# way if someone is typing away when a notice pops up, the user will
#		# find it far more difficult to enter the wrong thing into the popup
#		while 1:
#			pid = os.fork()
#			if pid == 0:
#				p,v = wait(pid)
#			else:
#				r=os.system(s)
#			sys.stderr.write('return value is %d\n' % r)
#			if r == None:
#				r = 0
#			else:
#				r = r / 256
#			sys.stderr.write('return value is %d\n' % r)
#			if r:
#				break
#		return

	def accident_proof_acknowledge(self,text,width,height):
		#dialog  --defaultno --yesno 'I know the process has completed' 30 80 ; echo $?
		text = 'In an Xdialog, you can select yes to exit.\nIn a dialog in a terminal emulator, you must destroy this window to continue!\n\n' + text
		s=self.prog+' --defaultno --yesno \''+text+'\' '+\
			self.__scale(height)+' '+self.__scale(width)
		# keep looping until the user selects "yes".  Default is no.  This
		# way if someone is typing away when a notice pops up, the user will
		# find it far more difficult to enter the wrong thing into the popup
		while 1:
			r=os.system(s)
			if r == None:
				r = 0
			else:
				r = r / 256
			if r == 0:
				break
		return

	def yesno(self,text,height,width,defaultno=0):
		# dialog doesn't have --defaultno.  Xdialog does.
#		if defaultno and self.x:
		s=self.prog+' --defaultno --yesno \''+text+'\' '+\
			self.__scale(height)+' '+self.__scale(width)
		r=os.system(s)
#		else:
#			s=self.prog+' --yesno \''+text+'\' '+self.__scale(height)+' '+\
#				self.__scale(width)
#			r=os.system(s)
		return r/256

	def msgbox(self,text,height,width):
		os.system(self.prog+' --cr-wrap --msgbox \''+text+'\' '+ \
			self.__scale(height)+' '+self.__scale(width))

# same as msgbox without a button (in text, not gui)?  If so, not a good thing.
#	def infobox(self,text,height,width):
#		os.system(self.prog+' --infobox '+text+' '+self.__scale(height)+' '+\
#			self.__scale(width))
	
	# doesn't work very well with the gdialog that comes with redhat 7.1.  Seems
	# fine with dialog and Xdialog.
	def textbox(self,file,height,width):
		os.system(self.prog+' --textbox \''+file+'\' '+self.__scale(height)+' '+\
			self.__scale(width))

	def menu(self,text,height,width,*options):
		l = len(options)
		if (l/2)*2 <> l:
			sys.stderr.write('incorrect arguments to menu method\n')
			sys.exit(0)
		#menulen = options[0]
		#options = options[1:]
		l = len(options)
		l2 = l / 2
		if l <> l2 * 2:
			sys.stderr.write('incorrect number of menu items in menu\n')
			sys.exit(0)
		menulen = l2
		cmd = self.prog+' --menu \''+text+'\' '+self.__scale(height)+' '+\
			self.__scale(width)+' '+str(menulen)
		i = 0
		while 1:
			if i >= l:
				break
			cmd = cmd + ' \''+options[i]+'\' \''+options[i+1]+'\''
			i = i + 2
		cmd = cmd + ' 2>&1'
		p = os.popen(cmd,'r')
		r = p.readline()
		p.close()
		#print r
		#print '-->',self.__nonl(r),'<--'
		return self.__nonl(r)

	def checklist(self,text,height,width,*options):
		#checklistlen = options[0]
		#options = options[1:]
		l = len(options)
		l3 = l / 3
		if l3*3 <> l:
			sys.stderr.write('incorrect arguments to checklist method\n')
			sys.exit(0)
		checklistlen = l3
		l = len(options)
		if l <> checklistlen*3:
			sys.stderr.write('incorrect number of checklist items in checklist\n')
			sys.exit(0)
		cmd = self.prog+' --separate-output --checklist \''+text+'\' '+\
			self.__scale(height)+' '+self.__scale(width)+' '+str(checklistlen)
		i = 0
		while 1:
			if i >= l:
				break
			cmd = cmd + ' \''+options[i]+'\' \''+options[i+1]+'\' \''+\
				options[i+2]+'\''
			i = i + 3
		cmd = cmd + ' 2>&1'
		p = os.popen(cmd,'r')
		lines = p.readlines()
		p.close()
		r = []
		for line in lines:
			r.append(self.__nonl(line))
		return r

	def radiolist(self,text,height,width,*options):
		#radiolistlen = options[0]
		#options = options[1:]
		l = len(options)
		l3 = l / 3
		if l3*3 <> l:
			sys.stderr.write('incorrect arguments to radiolist method\n')
			sys.exit(0)
		radiolistlen = l3
		l = len(options)
		if l <> radiolistlen*3:
			sys.stderr.write('incorrect number of radiolist items in radiolist\n')
			sys.exit(0)
		cmd = self.prog+' --radiolist \''+text+'\' '+\
			self.__scale(height)+' '+self.__scale(width)+' '+str(radiolistlen)
		i = 0
		while 1:
			if i >= l:
				break
			cmd = cmd + ' \''+options[i]+'\' \''+options[i+1]+'\' \''+\
				options[i+2]+'\''
			i = i + 3
		cmd = cmd + ' 2>&1'
		p = os.popen(cmd,'r')
		line = p.readline()
		p.close()
		return self.__nonl(line)

	def inputbox(self,text,height,width,init=''):
		cmd = self.prog+' --inputbox \''+text+'\' '+ \
			self.__scale(height)+' '+self.__scale(width)+' 2>&1'
		p=os.popen(cmd,'r')
		line = p.readline()
		p.close()
		r = self.__nonl(line)
		return r

	# gauge and guage don't look that convenient for the user, so no methods

def test():

	import pdialog
	
	p = pdialog.pdialog()

	print 'yesno'
	print p.yesno('yesno',20,60)

	print 'yesno --defaultno'
	print p.yesno('yesno --defaultno',20,60,1)

	print 'msgbox'
	p.msgbox('msgbox\nline2\nline3',20,60)

	#print 'infobox'
	#p.infobox('infobox',200,200)

	print 'textbox'
	p.textbox('/etc/passwd',24,60)

	print 'menu'
	print p.menu('menu',20,40,'a1','a2','b1','b2','c1','c2')

	print 'checklist'
	print p.checklist('checklist',20,60,'a1','a2','on','b1','b2','off',\
		'c1','c2','on')

	print p.radiolist('radiolist',20,60,'a1','a2','off','b1','b2','on',\
		'c1','c2','off')

	print p.inputbox('inputbox',20,60,'init')

if __name__ == '__main__':
	test()

